<?php

namespace Monarch;

/**
 * Provides color utilities for converting colors.
 */
class Color {

  /**
   * Get contrast ratio between two hex colors.
   *
   * @param string $hex1
   *   The first hex color to compare.
   * @param string $hex2
   *   The second hex color to compare.
   *
   * @return float|null
   *   The contrast ratio for the colors, or NULL if invalid colors were used.
   */
  public static function colorContrast(string $hex1, string $hex2) : ?float {
    $hex1 = strtolower($hex1);
    $hex2 = strtolower($hex2);

    if (
      preg_match('/#+([a-z0-9]{6,6}|[a-z0-9]{3,3})/', $hex1, $hex1matches) &&
      preg_match('/#+([a-z0-9]{6,6}|[a-z0-9]{3,3})/', $hex2, $hex2matches)
    ) {
      /** @var array $c */
      $c = [$hex1matches[1], $hex2matches[1]];

      foreach (array_keys($c) as $i) {
        if (strlen($c[$i]) === 6) {
          preg_match_all('/../', $c[$i], $c[$i]);

          foreach ($c[$i][0] as $j => $hex) {
            $c[$i][$j] = ((float) hexdec($hex)) / 0xff;
          }
        }
        else {
          preg_match_all('/./', $c[$i], $c[$i]);

          foreach ($c[$i][0] as $j => $hex) {
            $c[$i][$j] = ((float) hexdec($hex)) / 0xf;
          }
        }
      }

      foreach ($c as $i => $color) {
        foreach ($color as $j => $comp) {
          $c[$i][$j] = $comp < 0.03928 ? ($comp / 12.92) : pow(($comp + 0.055) / 1.055, 2.4);
        }

        $c[$i] = 0.2126 * $c[$i][0] + 0.7152 * $c[$i][1] + 0.0722 * $c[$i][2];
      }

      $light = max(...$c);
      $dark = min(...$c);

      return ($light + 0.05) / ($dark + 0.05);
    }

    return NULL;
  }

  /**
   * Convert color from hsl to hex.
   *
   * @param float $hue
   *   The color hue in degrees.
   * @param float $saturation
   *   The saturation of the color.
   *   Ranges from 0.0 to 1.0.
   * @param float $lightness
   *   The lightness of the color.
   *   Ranges from 0.0 to 1.0.
   *
   * @return string
   *   The hex string of the color.
   */
  public static function hslToHex(float $hue, float $saturation, float $lightness) : string {
    $hue = (($hue % 360) + 360) % 360;
    $saturation = max(min($saturation, 1), 0);
    $lightness = max(min($lightness, 1), 0);

    $c = (1 - abs(2 * $lightness - 1)) * $saturation;
    $x = $c * (1 - abs(($hue / 60) % 2 - 1));
    $m = $lightness - $c / 2;

    [$dr, $dg, $db] = [
      [$c, $x, 0],
      [$x, $c, 0],
      [0, $c, $x],
      [0, $x, $c],
      [$x, 0, $c],
      [$c, 0, $x],
    ][(int) ($hue / 60)];

    return '#' . implode('', [
      str_pad(dechex((int) ($dr + $m) * 255), 2, '0', STR_PAD_LEFT),
      str_pad(dechex((int) ($dg + $m) * 255), 2, '0', STR_PAD_LEFT),
      str_pad(dechex((int) $db + $m * 255), 2, '0', STR_PAD_LEFT),
    ]);
  }

}
